;; Copyright 2007, 2008, 2009, 2010, 2012, 2017 Kevin Ryde.   -*- coding: latin-1 -*-

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.



(progn
  (add-to-list 'load-path "..")
  (require 'tty-format))
(load (expand-file-name "../tty-format.el"))

(add-hook 'find-file-hooks 'tty-format-guess)

(progn
  (remove-hook 'find-file-hook 'tty-format-guess)
  (remove-hook 'find-file-hooks 'tty-format-guess))

(let ((tempfile "/tmp/x.el")
      (print-level nil))
  (require 'autoload)
  (find-file tempfile)
  (erase-buffer)
  (generate-file-autoloads "../tty-format.el")
  (save-buffer)
  (byte-compile-file tempfile)
  (delete-file (concat tempfile "c"))
  (delete-file tempfile))

(customize-face 'tty-format-overline)

;;-----------------
;; both


(and (memq 'utf-8 (coding-system-list))
     (decode-coding-string (string 226 128 190) 'utf-8))

(and (memq 'utf-8 (coding-system-list))
     (decode-coding-string (string-make-unibyte (string 226 128 190) 'utf-8))

(decode-char 'ucs 8254)

tty-format-utf8-overline-bytes

;;-----------------
;; both

(progn
  ;; (auto-compression-mode 1)
  (or (coding-system-p 'utf-8) ;; xemacs
      (require 'un-define))
  (modify-coding-system-alist 'file "man-db-manual\\.txt\\'" 'utf-8)
  (find-file "man-db-manual.txt")
  (format-decode-buffer 'backspace-overstrike)
  (format-decode-buffer 'ansi-colors))

;;-----------------
;; backspaces

(progn
  ;; (find-file "~/lisp/test/meintro.txt")
  (find-file "/usr/share/doc/groff-base/meintro.txt.gz")
  (format-decode-buffer 'backspace-overstrike))

(progn
  (auto-compression-mode 1)
  (find-file "/usr/share/doc/groff-base/meintro.txt.gz")

;;  (find-file "/usr/share/doc/libgtk1.2-doc/gtk_tut.txt.gz")
;;  (find-file "/usr/share/doc/libgtk1.2-doc/gtk_tut_fr.txt.gz")
;;  (find-file "/usr/share/doc/bind/bog.txt.gz")
  )

(require 'tty-format)
(setq debug-on-error t)
(locate-library "ansi-color")

;;-----------------
;; ansi

(progn
  (find-file "d-tty-format--curses.txt")
  (format-decode-buffer 'ansi-colors))

(progn
  (auto-compression-mode 1)
  ;; ansi-only
  (find-file "/usr/share/doc/libslang2/slang.txt.gz")
  ;; gone
  ;; (find-file "/usr/share/doc/bsd-mailx/manual.txt.gz")
  )
(progn
  (auto-compression-mode 1)
  ;; ansi plus overstrike bullet
  (find-file "/usr/share/doc/xserver-xorg-input-mouse/README.gz"))

(progn
  (auto-compression-mode 1)
  (find-file "/usr/share/doc/gnupg-doc/mini-HOWTO/GPGMiniHowto.txt.gz")

  ;; ansi plus overstrike daggar "|\b="
  ;; gone
  ;; (find-file "/usr/share/doc/bind/bog.txt.gz")
  )

(customize-variable 'find-file-hook)
(customize-variable 'find-file-hooks)


;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))


;;-----------------

;; big grep looking for ansi
(compile "locate /usr/share/doc | fgrep -v .tar | fgrep -v .dvi | fgrep -v .xls | fgrep -v .pdf | fgrep .gz |xargs zgrep '\033\\['")


;; big grep looking for backspace
(compile "locate /usr/share/doc | fgrep -v .tar | fgrep -v .dvi | fgrep -v .xls | fgrep -v .pdf | fgrep .gz |xargs zgrep '\b'")


(setq debug-on-error nil)


(progn
  (switch-to-buffer "*t-tty-format--test*")
  (erase-buffer)
  (insert "\033[1m\033[4mbold+underline\033[0;10m")
  (ansi-format-decode (point-min) (point-max))
  (describe-char (point-min)))

(progn
  (switch-to-buffer "*t-tty-format--test*")
  (erase-buffer)
  (insert "\033[1m\033[31m  red")
  (ansi-format-decode (point-min) (point-max))
  (describe-char (point-min)))



;;-----------------

(re-search-forward "_\b.\b.")
(re-search-forward "\\(.\\)\b_\b\\1")


(progn
  (switch-to-buffer "*t-tty-format--test*")
  (insert "a")
  (insert 8)
  (insert "A")
  (format-decode-buffer 'backspace-overstrike))


ansi sequences, plus ^H

groff tty char sources
    /usr/share/groff/1.18.1/tmac/tty-char.tmac
    /usr/share/groff/1.18.1/tmac/tty.tmac

U+2022
U+2023
U+2043
U+2219
U+29BF


;; The following regexp was an idea to get automatic detection.  It demands
;; three X\bX or _\bX sequences in a row.  But it's really still too
;; dangerous to let that lose on every arbitrary file read in.  (The havoc
;; caused by the changes backspace-overstrike-decode makes could be very
;; hard to track down.)
;;
;; "\\(.*\n\\)*.*\\(\\([^\b]\\)\b\\3\\([^\b]\\)\b\\4\\([^\b]\\)\b\\5\\|_\b[^\b]_\b[^\b]_\b[^\b]\\)"


<p>
For reference, here are some other programs that can cope with this sort of
backspace overstriking
<ul>
<li> <code>col -b</code> (the standard unix tool) or <code>colcrt</code>
     &mdash; can flatten them out
<li> <a href="http://www.greenwoodsoftware.com/less/">less</a> &mdash; shows
     them as bold and/or underline
</ul>


;;
;; Other programs handling this include
;;
;;     - "less" shows such sequences as bold and underline
;;
;;     - "ul" converts to curses terminal control sequences
;;
;;     - "col -b" keeps just the last just the last char in backspace
;;       overstrike.
;;
;;     - "colcrt" converts underscores to dashes on an extra line


(progn
  (facemenu-add-face 'bold (point) (1+ (point)))
  (facemenu-add-face 'underline (point) (1+ (point)))
  (describe-char (point)))
x

(facemenu-active-faces '(bold underline))

(progn
  (insert (format "%s" (face-attributes-as-vector 'bold)))
  (insert "\n")
  (insert (format "%s" (face-attributes-as-vector 'underlinee))))
[unspecified unspecified unspecified unspecified bold unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified nil nil unspecified]
[unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified unspecified]


(ansi-color-apply-sequence "\033[1m" nil)
(ansi-color-apply-sequence "\033[1m" '(default))
(ansi-color-apply-sequence "\033[1m" 'underline)


(insert (propertize "x" 'face '(bold)))x
(insert (propertize "x" 'face '(bold underline)))x

(insert (propertize "x" 'face '(overline)))x

(insert (propertize "x" 'face 'tty-overline))x

(decode-coding-string (string-make-unibyte (string #xE2 #x80 #xBE)) 'utf-8)
(require 'un-define)
(decode-coding-string tty-format-utf8-overline-bytes 'utf-8)
            

(facep 'bold)
(facep 'underline)
(facep 'overline)
(facep 'tty-overline)
(facep 'underline-bold)

(progn
  (require 'cl)
  (remove-if (lambda (sym)
               (let ((str (symbol-name sym)))
                 (not (string-match "overline" str))))
             (face-list)))

(defface my-overline
		'((t :overline t))
		"An overline face."
		:group 'my-faces)
(featurep 'cus-face)
(macroexpand '(defface my-overline
		'((t :overline t))
		"An overline face."
		:group 'my-faces))
custom-declare-face

(defface)
custom-face-attributes

;; no decode-char in xemacs
(decode-char 'ucs #x203E)
