#!/usr/bin/perl -w

# Copyright 2008, 2009, 2017 Kevin Ryde.
#
# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use FindBin;
use File::Slurp;

use lib::abs $FindBin::Bin;
use MyStuff;

foreach my $filename (glob("$FindBin::Bin/../../*/*.el"),
                      glob("$FindBin::Bin/../../../pchart/emacs/*.el")) {
  my $wip_filename = $filename;
  if ($wip_filename =~ s{(.*)/}{$1/wip/}) {
    if (-e $wip_filename) {
      $filename = $wip_filename;
    }
  }

  print "$filename\n";
  my $str = File::Slurp::read_file($filename);

  $str =~ s/;.*//mg;

  while ($str =~ /\n\(defun *([^ ]+).*?"(.*?)"(.*?)\n(\(|$)/sg) {
    my $name = $1;
    my $docstring = $2;
    my $body = $3;
    next if $name =~ /unload-function/;
    next if $docstring =~ /an internal/i;
    next if $body =~ /interactive/i;

    my ($linenum, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]);
    print "$filename:$linenum: $name\n";

    pos($str) = pos($str) - 2;
  }
}
exit 0;
