#!/usr/bin/perl -w

# Copyright 2009, 2010 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-perl-use-utf8-coding.pl
#
# Find perl files containing "use utf8" to specify their coding system.
#

use strict;
use warnings;
use Perl6::Slurp;

use lib::abs '.';
use MyLocatePerl;
use MyStuff;

my $verbose = 0;

my $l = MyLocatePerl->new;
while (my ($filename,$str) = $l->next) {
  if ($verbose) { print "look at $filename\n"; }

  next unless ($str =~ /use utf8/m);
  my $pos = $-[0];

  # to skip the ones with a ";" -- none found without
  #next if ($str =~ /^use utf8[;]/m);

  # to see only indented ones
  #next if ($str =~ /^use utf8/m);

  my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
  print "$filename:$line:$col: directive\n", MyStuff::line_at_pos($str, $pos);

  if ($str =~ /coding\s*:/) {
    $pos = $-[0];
    ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
    print "$filename:$line:$col: coding cookie\n",
      MyStuff::line_at_pos($str,$-[0]);
  }

  if ($str =~ m/[^\0-\177]/) {
    $pos = $-[0];
    ($line, $col) = MyStuff::pos_to_line_and_column ($str, $pos);
    print "$filename:$line:$col: multibyte\n";

  }
}

exit 0;
