;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(require 'cl)

(defun my-count-args ()
  (save-excursion
    (let ((count 0))
      (while (condition-case nil
                 (goto-char (scan-sexps (point) 1))
               (error nil))
        (incf count))
      count)))

(put 'my-grep-buffer emacs-version 123)

(defun my-grep-buffer ()
  (save-excursion
    (goto-char (point-min))
    (catch 'return
      (while t
        (let ((pending (condition-case err
                           (list (read (current-buffer)))
                         (end-of-file
                          (throw 'return nil))
                         (error
                          (let ((line   (count-lines (point-min) (point)))
                                (column (current-column)))
                            (with-current-buffer (get-buffer-create "*grep*")
                              (insert (format "%s:%d:%d: error %S\n"
                                              filename line column
                                              (error-message-string err)))))
                          (redisplay t)
                          (throw 'return nil)))))
          (while pending
            (let ((obj (pop pending)))
              (cond ((consp obj)
                     (my-grep-form obj)
                     (while (consp obj)
                       (push (pop obj) pending))
                     (push obj pending))
                    ((vectorp obj)
                     (dotimes (i (length obj))
                       (push (elt obj i) pending)))))))))))
(byte-compile 'my-grep-buffer)

(defun my-grep-form (form)
  ;; (with-current-buffer (get-buffer-create "*grep*")
  ;;   (insert (format "form %S\n" obj)))
  (when (and (eq 'put (car-safe form))
             (equal 4 (safe-length form))
             (not (consp (nth 2 form)))
             (not (keywordp (nth 2 form))))
    (let (;; (filename buffer-file-name)
          (line     (count-lines (point-min) (point)))
          (column   (current-column)))
      (with-current-buffer (get-buffer-create "*grep*")
        (insert (format "%s:%s:%s: %S\n"
                        filename line column
                        form))
        (redisplay t)))))

(defun my-grep-file (filename)
  (require 'lisp-mnt)
  (lm-with-file filename
    (my-grep-buffer)))

(defun my-grep (&rest wildcards)
  (kill-buffer (get-buffer-create "*grep*"))
  (switch-to-buffer "*grep*")
  (insert "\n")
  (dolist (wildcard wildcards)
    (let ((filename-list (file-expand-wildcards wildcard)))
      (with-current-buffer (get-buffer-create "*grep*")
        (insert (format "%s %d\n" wildcard (length filename-list))))
      (redisplay t)
      (dolist (filename filename-list)
        (my-grep-file filename))))
  (insert "done\n")
  (goto-char (point-min))
  (compilation-mode))

(my-grep "grep-put-unquoted.el")
(my-grep "/down/w3m-el/emacs-w3m/w3m-util.el")

(my-grep "/down/emacs/lisp/*.el"
         "/down/emacs/lisp/*/*.el"
         "/down/emacs/lisp/*/*/*.el")

(my-grep "/usr/share/emacs/site-lisp/*.el"
         "/usr/share/emacs/site-lisp/*/*.el"
         "/usr/share/emacs/site-lisp/*/*/*.el")

(my-grep "/down/el/*.el")
(my-grep "/down/ding/gnus/lisp/*.el")

(my-grep "/usr/share/xemacs-21.4.22/lisp/*.el.gz"
         "/usr/share/xemacs-21.4.22/lisp/*/*.el.gz")
(my-grep "/usr/share/xemacs21/xemacs-packages/lisp*.el.gz"
         "/usr/share/xemacs21/xemacs-packages/lisp/*/*.el.gz")

                   ;; "/usr/share/emacs/site-lisp/*/*.el"



;;--------------------------
;; (add-hook 'kill-buffer-hook (lambda () (message "bye")))
