;; Copyright 2012 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------

<lang=lisp>(defun my-recurse (n)
  (my-recurse (1+ n)))
(my-recurse 1)
=>
enters debugger at (my-recurse 595),
which is the default max-lisp-eval-depth 600 in Emacs 24.1</lang>

Variable <code>max-lisp-eval-depth</code>[http://www.gnu.org/software/emacs/manual/html_node/elisp/Eval.html#index-max_002dlisp_002deval_002ddepth-539] is the maximum depth of function calls and variable <code>max-specpdl-size</code>[http://www.gnu.org/software/emacs/manual/html_node/elisp/Local-Variables.html#index-max_002dspecpdl_002dsize-614] is the maximum depth of nested <code>let</code> bindings.  A function call is a <code>let</code> of the parameters, even if there's no parameters, and so counts towards <code>max-specpdl-size</code> as well as <code>max-lisp-eval-depth</code>.

The limits can be increased with <code>setq</code> etc globally, or <code>let</code> etc temporarily.  Lisp code which knows it needs deep recursion might temporarily increase the limits.  Eg. <code>regexp-opt.el</code>.  The ultimate limit is memory or C stack.


(setq max-lisp-eval-depth 800)
(setq max-lisp-eval-depth 500)
max-lisp-eval-depth
max-specpdl-size

  (interactive)

(defun my-recurse ()
  (my-recurse))
(my-recurse)

(defun my-recurse (n m)
  (let (x)
    (my-recurse (1+ n) 9)))
(my-recurse 1 2)


(byte-compile 'my-recurse)

  (interactive "d")



;;-----------------------------------------------------------------------------

(directory-files "/some/dir/name"
                 nil        ;; just the filenames, not full paths
                 "\\.c\\'"  ;; regexp
                 t)         ;; don't sort the filenames

<code>directory-files</code> gives the filenames in a given
directory, optionally restricted to those matching a regexp.

(directory-files "."
                 nil        ;; just the filenames, not full paths
                 "\\.c\\'"  ;; regexp
                 t)         ;; don't sort the filenames


;;-----------------------------------------------------------------------------

(ding)    ;; ring the bell

(beep)    ;; the same thing

On a tty or in <code>-batch</code> mode this sends a BEL
character.  In a GUI it does whatever suits the window system.
Variables <code>visible-bell</code> and
<code>ring-bell-function</code> can control the behaviour.

<code>beep</code> was originally called <code>feep</code>, but
that changed recently :-)

Fri Dec 13 00:52:16 1985  Richard M. Stallman  (rms at prep)
	* subr.el: Rename feep to beep, a more traditional name.

;;-----------------------------------------------------------------------------

<code>message</code> displays the numbers in the message area, or
if run from <code>-batch</code> it prints to stderr.

(dotimes (i most-positive-fixnum) ;; 0 to mostpos-1
  (message "%d" (1+ i)))

<code>calc.el</code> can do arbitrary precision.

;; `math-add' automatically promotes to a calc.el bignum when necessary
;; (which is at `math-small-integer-size' or more)
(require 'calc)
(let ((i 0))
  (while t
    (setq i (math-add i 1))  ;; increment
    (message "%s" (math-format-number i))))











(math-add (1- math-small-integer-size) 0)
(math-add most-positive-fixnum most-positive-fixnum)
(math-format-number 0)
