;; Copyright 2013 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-mailcap-mode--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(mailcap-mode))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; mailcap-mode-syntax-table

    (with-temp-buffer
      (mailcap-mode)

      (mapcar (lambda (elem)
                (let ((char (nth 0 elem))
                      (want (nth 1 elem)))
                  (let ((got (char-syntax char)))
                    (unless (equal got want)
                      (error "char %c syntax want %c got %c" char want got))
                    (setq count (1+ count)))))
              '((?\\ ?\\)
                (?\" ?\")
                (?\' ?\")
                (?#  ?<)
                (?\n ?>)
                )))

    ;;-------------------------------------------------------------------------
    ;; mailcap-mode-beginning-of-entry

    (mailcap-mode)
    (mapcar (lambda (elem)
              (let ((str   (nth 0 elem))
                    (start (nth 1 elem))
                    (want  (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ start (point-min)))
                (while (not (eobp))
                  (let ((got (- (save-excursion
                                  (mailcap-mode-beginning-of-entry))
                                (point-min))))
                    (unless (equal want got)
                      (error "mailcap-mode-beginning-of-entry want %S got %S"
                             want got)))
                  (goto-char (1+ (point)))))

              (setq count (1+ count)))

            '(
              ("text/plain; cat %s\n"
               0 0)

              ("#x\\\ntext/plain; \\\ncat %s\\\n; needsterminal"
               4 4)
              ))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'mailcap-mode)
    (unload-feature 'mailcap-mode t)
    (require 'mailcap-mode)
    (unload-feature 'mailcap-mode t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))





;; ;;-------------------------------------------------------------------------
;; ;; mailcap-mode-beginning-of-entry-p
;; 
;; (mailcap-mode)
;; (mapcar (lambda (elem)
;;           (let ((str  (nth 0 elem))
;;                 (line (nth 1 elem))
;;                 (want (nth 2 elem)))
;;             (erase-buffer)
;;             (insert str)
;;             (goto-char (point-min))
;;             (forward-line line)
;;             (let ((got (not (not (mailcap-mode-beginning-of-entry-p)))))
;;               (unless (equal want got)
;;                 (error "mailcap-mode-beginning-of-entry-p line %S want %S got %S"
;;                        line want got)))
;;             (setq count (1+ count))))
;; 
;;         '(
;;           ("text/plain; cat %s\n"
;;            0 t)
;; 
;;           ("#x\\\ntext/plain; \\\ncat %s\\\n; needsterminal"
;;            0 t)
;;           ("#x\\\ntext/plain; \\\ncat %s\\\n; needsterminal"
;;            1 t)
;;           ("#x\\\ntext/plain; \\\ncat %s\\\n; needsterminal"
;;            2 nil)
;;           ("#x\\\ntext/plain; \\\ncat %s\\\n; needsterminal"
;;            3 nil)
;; 
;; 
;;           ("text/plain; \\\n# command\\\n#command\\\n; needsterminal"
;;            0 t)
;;           ("text/plain; \\\n# command\\\n#command\\\n; needsterminal"
;;            1 nil)
;;           ("text/plain; \\\n# command\\\n#command\\\n; needsterminal"
;;            2 nil)
;;           ("text/plain; \\\n# command\\\n#command\\\n; needsterminal"
;;            3 nil)
;;           ))
