;; Copyright 2015 Kevin Ryde

;; t-no-require-self.el is shared by several distributions.
;;
;; t-no-require-self.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; t-no-require-self.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this file.  If not, see <http://www.gnu.org/licenses/>.


(eval-when-compile
  (unless (fboundp 'dolist)
    (require 'cl))) ;; for `dolist' in emacs20

(defun my-contains-form (elem target)
  "Return non-nil if ELEM is a list containing TARGET, recursively."
  (let ((pending (list elem))
        found)
    (while pending
      (let ((x (car pending)))
        (setq pending (cdr pending))
        (cond ((equal x target)
               (setq found t)
               (setq pending nil))
              ((consp x)
               (setq pending (cons (car x)
                                   (cons (cdr x)
                                         pending)))))))
    found))

(unless (my-contains-form '(1 2 3) 2)
  (error "oops"))
(unless (my-contains-form '(require foo) '(require foo))
  (error "oops"))
(when (my-contains-form '(require bar) '(require foo))
  (error "oops"))
(unless (my-contains-form '(progn (progn (require foo))) '(require foo))
  (error "oops"))
(when (my-contains-form [1 2] '(require foo))
  (error "oops"))
(when (my-contains-form 99.99 '(require foo))
  (error "oops"))


(defun my-run-test ()
  (let* ((count 0)

         (srcdir (expand-file-name ".."))
         (file-list (directory-files srcdir nil "\\`[^.].*\\.el\\'"))

         (load-path (cons srcdir load-path))
         (find-function-source-path nil))

    (message "srcdir is %S" srcdir)
    (when (null file-list)
      (error "Oops, no *.el files"))

    (dolist (basename file-list)
      (let ((fullname (expand-file-name basename srcdir)))
        (message "check %S" basename)

        ;;-------------------------------------------------------------------
        ;; lm-authors

        (find-file fullname)
        (goto-char (point-min))
        (let ((target (list 'require (list 'quote (intern (substring basename 0 -3)))))
elem)
          ;; (message "%S" target)
          (while (condition-case err
                     (progn (setq elem (read (current-buffer))) t)
                   (end-of-file nil))
            (when (my-contains-form elem target)
              (error "%S contains %S" fullname target)
              (goto-char (point-max)))))
        (setq count (1+ count))

        ;;-------------------------------------------------------------------
        ;; the find-file
        (kill-buffer nil)))

    (message "  %d tests" count)
    count))
