;; Copyright 2016, 2017, 2020 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-sentence-spacing--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'sentence-spacing)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(sentence-spacing
              sentence-spacing-display))

    ;;-------------------------------------------------------------------------
    ;; sentence-spacing

    ;; no :upper: etc in xemacs
    (if (not (string-match "[[:alpha:]]" "A"))
        (message "no [[:alpha:]] etc regexps in this emacs")

      (mapcar (lambda (elem)
                (let ((input (nth 0 elem))
                      (want  (nth 1 elem)))
                  (erase-buffer)
                  (insert input)
                  (sentence-spacing)
                  (let ((got (buffer-string)))
                    (unless (equal want got)
                      (goto-char (point-max))
                      (erase-buffer)
                      (insert "\n---input---\n")
                      (insert input)
                      (insert "\n---got---\n")
                      (insert (format "%s\n" got))
                      (insert "\n---want---\n")
                      (insert want)
                      (error "wrong at %s\n%s\n" count got))))
                (setq count (1+ count)))

              '(
                ("Blah.!'? Blah" "Blah.!'?  Blah")
                ("Blah!). Blah." "Blah!).  Blah.")
                ("Blah?? Blah."  "Blah??  Blah.")
                ("Blah?! Blah."  "Blah?!  Blah.")
                ("Blah... Blah" "Blah...  Blah") ;; ellipsis still ends
                (". . . ." ". . . .")    ;; dots, not sentences
                ("... ... ... ..." "... ... ... ...")
                ("part a. or part b." "part a. or part b.")
                ("A.B. Foo" "A.B. Foo")
                ("blah. Blah" "blah.  Blah")
                ("Blah DVD. Blah blah." "Blah DVD.  Blah blah.")
                ("Blah!! Blah.."  "Blah!!  Blah..")
                ("ie. blah" "ie. blah")
                ("i.e. blah" "i.e. blah")
                ("Some B. One" "Some B. One")
                ("Mrs. Jones" "Mrs. Jones")

                ("(Parens.) Next." "(Parens.)  Next.")
                ("[Brackets.] Next." "[Brackets.]  Next.")
                ("\"Quotes.\" Next." "\"Quotes.\"  Next.")
                ("'Quotes.' Next." "'Quotes.'  Next.")
                )))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    ;; couple of cycles
    (require 'sentence-spacing)
    (unload-feature 'sentence-spacing t)
    (require 'sentence-spacing)
    (unload-feature 'sentence-spacing t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
